# (c) 2015, Jonathan Davila <jonathan(at)davila.io>
# (c) 2017 Ansible Project

from __future__ import (absolute_import, division, print_function)

EXAMPLES = """
  - name: Using PAM360
    debug:
      msg: "{{ lookup('pam360','ResourceName', 'AccountName') }}"
"""

RETURN = """
  _raw:
    description:
      - Password stored in PAM360
"""

import os
import json
import urllib

from ansible.errors import AnsibleError
from ansible.module_utils.parsing.convert_bool import boolean
from ansible.plugins.lookup import LookupBase

from ansible.module_utils.urls import open_url

PAM_URL = ''
PAM_AUTHTOKEN =''
VALIDATE_CERT=''

if os.getenv('PAM360_URL') is not None:
    PAM_URL = os.environ['PAM360_URL']
else:
    raise AnsibleError("PAM360 ERROR - Set PAM Url in environent variable");
if os.getenv('PAM360_AUTHTOKEN') is not None:
    PAM_AUTHTOKEN = os.environ['PAM360_AUTHTOKEN']
else:
    raise AnsibleError("PAM360 ERROR - Set Authtoken in environent variable");
if os.getenv('VALIDATE_PAM360_CERT') is not None:
    VALIDATE_CERT = os.environ['VALIDATE_PAM360_CERT']
    if VALIDATE_CERT.lower() == 'false':
        VALIDATE_CERT = False
    else:
        VALIDATE_CERT = True
else:
    VALIDATE_CERT = True



def getPAMResourceAccountDetails(resourceId,accountId):
    url = '{0}/restapi/json/v1/resources/getResourceAccountDetails?APP_NAME=ANSIBLE&RESOURCEID={1}&ACCOUNTID={2}'.format(PAM_URL, resourceId , accountId)  
    headers = {'Content-Type':'application/json','APP_AUTHTOKEN':PAM_AUTHTOKEN}
    response = open_url(url, method='GET' ,headers=headers,validate_certs=VALIDATE_CERT)
    resp_json = json.loads(response.read().decode('utf-8'))
    if resp_json.get("operation").get("Details"):
       details = resp_json.get("operation").get("Details")
       resourceDetails = details.get("RESOURCEDETAILS")
       accountDetails = json.loads(details.get("ACCOUNTDETAILS"))
       if accountDetails.get("PASSWORD_REASON"):
          raise AnsibleError("Couldn't able to read password from PAM360 : ",accountDetails.get("PASSWORD_REASON"))
       else:						  
          password = accountDetails.get("PASSWORD")   
          return [password]    
    else:
       message = resp_json.get("operation").get("result").get("message")
       raise AnsibleError("PAM360 ERROR -",message)
       
   
class LookupModule(LookupBase):

    def run(self, terms, variables=None, **kwargs):
        resourceName = terms[0]
        accountName = terms[1]
        url = '{0}/restapi/json/v1/resources/getResourceIdAccountId?APP_NAME=Ansible&RESOURCENAME={1}&ACCOUNTNAME={2}'.format(PAM_URL, urllib.parse.quote(resourceName) , urllib.parse.quote(accountName))  
        headers = {'Content-Type':'application/json','APP_AUTHTOKEN':PAM_AUTHTOKEN} 
        response = open_url(url, method='GET' ,headers=headers,validate_certs=VALIDATE_CERT)
        resp_json = json.loads(response.read().decode('utf-8'))
        resp_json = resp_json.get("operation");
        if resp_json.get("Details"):
          details = resp_json.get("Details")
          resourceId = details.get("RESOURCEID")
          accountId = details.get("ACCOUNTID")
          password = getPAMResourceAccountDetails(resourceId , accountId)
          return password        
        else:
          message = resp_json.get("result").get("message")
          raise AnsibleError("PAM360 ERROR - %s " % (message))
